@@ -1,4 +1,3 @@
-#!/usr/bin/python
 ###############################################################################
 #                                                                             #
 # Fireinfo                                                                    #
@@ -24,13 +23,12 @@ import json
 import os
 import string
 
-import _fireinfo
-
-import bios
-import cpu
-import device
-import hypervisor
-import network
+from . import _fireinfo
+from . import bios
+from . import cpu
+from . import device
+from . import hypervisor
+from . import network
 
 PROFILE_VERSION = 0
 
@@ -42,6 +40,7 @@ INVALID_ID_STRINGS = (
 	"N/A", "n/a",
 	"12345", "54321", "202020",
 	"Chassis", "chassis",
+	"Default string",
 	"EVAL",
 	"Not Applicable",
 	"None", "empty",
@@ -83,9 +82,7 @@ def read_from_file(filename):
 	except IOError:
 		pass
 
-class System(object):
-	__metaclass__ = Singleton
-
+class System(object, metaclass=Singleton):
 	def __init__(self):
 		self.bios = bios.BIOS(self)
 
@@ -192,7 +189,8 @@ class System(object):
 		if not public_id:
 			return "0" * 40
 
-		return hashlib.sha1(public_id).hexdigest()
+		h = hashlib.sha1(public_id.encode())
+		return h.hexdigest()
 
 	@property
 	def private_id(self):
@@ -209,7 +207,8 @@ class System(object):
 		if not private_id:
 			return "0" * 40
 
-		return hashlib.sha1(private_id).hexdigest()
+		h = hashlib.sha1(private_id.encode())
+		return h.hexdigest()
 
 	@property
 	def secret_id(self):
@@ -220,7 +219,8 @@ class System(object):
 		if os.path.exists(SECRET_ID_FILE):
 			return read_from_file(SECRET_ID_FILE)
 
-		return hashlib.sha1(self._unique_id).hexdigest()
+		h = hashlib.sha1(self._unique_id.encode())
+		return h.hexdigest()
 
 	@property
 	def _unique_id(self):
@@ -337,7 +337,7 @@ class System(object):
 		if s is None:
 			return
 
-		return filter(lambda x: x in string.printable, s)
+		return "".join([x for x in s if x in string.printable])
 
 	@property
 	def vendor(self):
@@ -372,7 +372,7 @@ class System(object):
 		ret = read_from_file("/proc/device-tree/model")
 		if ret:
 			# replace the NULL byte with which the DT string ends
-			ret = ret.replace(u"\u0000", "")
+			ret = ret.replace("\u0000", "")
 
 		# Fall back to read /proc/cpuinfo
 		if not ret:
@@ -401,21 +401,26 @@ class System(object):
 		"""
 			Return the dev node of the root disk.
 		"""
-		with open("/etc/mtab", "r") as f:
-			dev, mountpoint, fs, rest = f.readline().split(" ", 3)
-			if mountpoint == "/" and not fs == "rootfs":
-				# Cut off /dev
-				dev = dev[5:]
+		with open("/proc/mounts", "r") as f:
+			for line in f.readlines():
+				# Skip empty lines
+				if not line:
+					continue
 
-				# Handle raids and MMC cards like (mmcblk0p3).
-				if dev[-2] == "p":
-					return dev[:-2]
+				dev, mountpoint, fs, rest = line.split(" ", 3)
+				if mountpoint == "/" and not fs == "rootfs":
+					# Cut off /dev
+					dev = dev[5:]
 
-				# Otherwise cut off all digits at end of string
-				while dev[-1] in string.digits:
-					dev = dev[:-1]
+					# Handle raids and MMC cards like (mmcblk0p3).
+					if dev[-2] == "p":
+						return dev[:-2]
 
-				return dev
+					# Otherwise cut off all digits at end of string
+					while dev[-1] in string.digits:
+						dev = dev[:-1]
+
+					return dev
 
 	@property
 	def root_size(self):
@@ -479,13 +484,13 @@ class System(object):
 
 if __name__ == "__main__":
 	s=System()
-	print s.arch
-	print s.language
-	print s.release
-	print s.bios_vendor
-	print s.memory
-	print s.kernel
-	print s.root_disk
-	print s.root_size
-	print "------------\n", s.devices, "\n------------\n"
-	print json.dumps(s.profile(), sort_keys=True, indent=4)
+	print(s.arch)
+	print(s.language)
+	print(s.release)
+	print(s.bios_vendor)
+	print(s.memory)
+	print(s.kernel)
+	print(s.root_disk)
+	print(s.root_size)
+	print("------------\n", s.devices, "\n------------\n")
+	print(json.dumps(s.profile(), sort_keys=True, indent=4))
